/******************************************************************************
* Copyright (C) 2018 Xilinx, Inc.  All rights reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* Use of the Software is limited solely to applications:
* (a) running on a Xilinx device, or
* (b) that interact with a Xilinx device through a bus or interconnect.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
* XILINX  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
* OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* Except as contained in this notice, the name of the Xilinx shall not be used
* in advertising or otherwise to promote the sale, use or other dealings in
* this Software without prior written authorization from Xilinx.
******************************************************************************/

#include "xparameters.h"
#include "xpfw_core.h"
#include "xpfw_default.h"
#include "xpfw_module.h"
#include "xstatus.h"
#include "xsysmonpsu.h"


#include "xpfw_mod_sysmon_psu.h"

const XPfw_Module_t *SysmonPsModPtr;

/*
* Instance of SYSMON Driver
*/

static XSysMonPsu SysMonInst;
static XSysMonPsu *SysMonInstPtr = &SysMonInst;

/*
* PS-SYSMON voltage/temperature read time in milliseconds.
*/
#define XPFW_PS_SYSMON_VOLTAGE_TEMP_READ_TIME 10000U

/****************************************************************************/
/**
*
* This function converts the fraction part of the given floating point number
* (after the decimal point)to an integer.
*
* @param    FloatNum is the floating point number.
*
* @return   Integer number to a precision of 3 digits.
*
* @note
* This function is used in the printing of floating point data to a STDIO device
* using the xil_printf function. The xil_printf is a very small foot-print
* printf function and does not support the printing of floating point numbers.
*
*****************************************************************************/

static int SysMonPsuFractionToInt(float FloatNum)
{
	float Temp;

	Temp = FloatNum;
	if (FloatNum < 0) {
	Temp = -(FloatNum);
	}

	return( ((int)((Temp -(float)((int)Temp)) * (1000.0f))));
}


/****************************************************************************/
/**
*
* This function continuously read the latest temperatures and voltages
*  and prints
*
*/
static void SysmonPsuPolledExamplePrint(void)
{
	u32 TempRawData;
	u32 VccRawData;
	float TempData;
	float VccData;
	float MaxData;
	float MinData;

	/* Wait till the End of Sequence occurs */
	while ((XSysMonPsu_IntrGetStatus(SysMonInstPtr) & ((u64)XSYSMONPSU_ISR_1_EOS_MASK<< 32)) !=
	((u64)XSYSMONPSU_ISR_1_EOS_MASK<< 32));

	/*
	* Read the on-chip Temperature Data (Current/Maximum/Minimum)
	* from the ADC data registers.
	*/

	TempRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_TEMP, XSYSMON_PS);
	TempData = XSysMonPsu_RawToTemperature_OnChip(TempRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"\r\nThe Current LP Temperature is %0d.%03d Centigrades.\r\n",
	(int)(TempData), SysMonPsuFractionToInt(TempData));


	TempRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MAX_TEMP, XSYSMON_PS);
	MaxData = XSysMonPsu_RawToTemperature_OnChip(TempRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"The Maximum LP Temperature is %0d.%03d Centigrades. \r\n",
	(int)(MaxData), SysMonPsuFractionToInt(MaxData));

	TempRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MIN_TEMP, XSYSMON_PS);
	MinData = XSysMonPsu_RawToTemperature_OnChip(TempRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"The Minimum LP Temperature is %0d.%03d Centigrades. \r\n",
	(int)(MinData), SysMonPsuFractionToInt(MinData));


	TempRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_TEMP_REMTE, XSYSMON_PS);
	TempData = XSysMonPsu_RawToTemperature_OnChip(TempRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"\r\nThe Current FP Temperature is %0d.%03d Centigrades.\r\n",
	(int)(TempData), SysMonPsuFractionToInt(TempData));


	TempRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MAX_TEMP_REMOTE, XSYSMON_PS);
	MaxData = XSysMonPsu_RawToTemperature_OnChip(TempRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"The Maximum FP Temperature is %0d.%03d Centigrades. \r\n",
	(int)(MaxData), SysMonPsuFractionToInt(MaxData));

	TempRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MIN_TEMP_REMOTE, XSYSMON_PS);
	MinData = XSysMonPsu_RawToTemperature_OnChip(TempRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"The Minimum FP Temperature is %0d.%03d Centigrades. \r\n",
	(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	* Read the SUPPLY1 VCC_PSINTLP Voltage Data (Current/Maximum/Minimum) from the
	* ADC data registers.
	*/
	VccRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_SUPPLY1, XSYSMON_PS);
	VccData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"\r\nThe Current VCCINTLP is %0d.%03d Volts. \r\n",
	(int)(VccData), SysMonPsuFractionToInt(VccData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MAX_SUPPLY1, XSYSMON_PS);
	MaxData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"The Maximum VCCINTLP is %0d.%03d Volts. \r\n",
	(int)(MaxData), SysMonPsuFractionToInt(MaxData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MIN_SUPPLY1, XSYSMON_PS);
	MinData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"The Minimum VCCINTLP is %0d.%03d Volts. \r\n",
	(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	* Read the SUPPLY2 VCC_PSINTFP Voltage Data (Current/Maximum/Minimum) from the
	* ADC data registers.
	*/
	VccRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_SUPPLY2, XSYSMON_PS);
	VccData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"\r\nThe Current VCCINTFP is %0d.%03d Volts. \r\n",
	(int)(VccData), SysMonPsuFractionToInt(VccData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MAX_SUPPLY2, XSYSMON_PS);
	MaxData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"The Maximum VCCINTFP is %0d.%03d Volts. \r\n",
	(int)(MaxData), SysMonPsuFractionToInt(MaxData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MIN_SUPPLY2, XSYSMON_PS);
	MinData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"The Minimum VCCINTFP is %0d.%03d Volts. \r\n",
	(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	* Read the SUPPLY3 VCC_PSAUX Votage Data (Current/Maximum/Minimum) from the
	* ADC data registers.
	*/
	VccRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_SUPPLY3, XSYSMON_PS);
	VccData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"\r\nThe Current VCC_PSAUX is %0d.%03d Volts. \r\n",
	(int)(VccData), SysMonPsuFractionToInt(VccData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MAX_SUPPLY3, XSYSMON_PS);
	MaxData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Maximum VCC_PSAUX is %0d.%03d Volts. \r\n",
	(int)(MaxData), SysMonPsuFractionToInt(MaxData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MIN_SUPPLY3, XSYSMON_PS);
	MinData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"The Minimum VCC_PSAUX is %0d.%03d Volts. \r\n",
	(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	* Read the SUPPLY4 VCC_PSDDR_504 Voltage Data (Current/Maximum/Minimum) from the
	* ADC data registers.
	*/
	VccRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_SUPPLY4, XSYSMON_PS);
	VccData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"\r\nThe Current VCC_PSDDR is %0d.%03d Volts. \r\n",
	(int)(VccData), SysMonPsuFractionToInt(VccData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MAX_SUPPLY4, XSYSMON_PS);
	MaxData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Maximum VCC_PSDDR is %0d.%03d Volts. \r\n",
	(int)(MaxData), SysMonPsuFractionToInt(MaxData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MIN_SUPPLY4, XSYSMON_PS);
	MinData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"The Minimum VCC_PSDDR is %0d.%03d Volts. \r\n",
	(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	* Read the VCCAMS Voltage Data (Current/Maximum/Minimum) from the
	* ADC data registers.
	*/
	VccRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_VCCAMS, XSYSMON_PS);
	VccData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"\r\nThe Current VCCAMS is %0d.%03d Volts. \r\n",
	(int)(VccData), SysMonPsuFractionToInt(VccData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MAX_VCCAMS, XSYSMON_PS);
	MaxData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Maximum VCCAMS is %0d.%03d Volts. \r\n",
	(int)(MaxData), SysMonPsuFractionToInt(MaxData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MIN_VCCAMS, XSYSMON_PS);
	MinData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"The Minimum VCCAMS is %0d.%03d Volts. \r\n",
	(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	* Read the SUPPLY5 VCC_PSIO3_503 Voltage Data (Current/Maximum/Minimum) from the
	* ADC data registers.
	*/
	VccRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_SUPPLY5, XSYSMON_PS);
	VccData = XSysMonPsu_VccopsioRawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"\r\nThe Current VCC_PSIO3 is %0d.%03d Volts. \r\n",
	(int)(VccData), SysMonPsuFractionToInt(VccData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MAX_SUPPLY5, XSYSMON_PS);
	MaxData = XSysMonPsu_VccopsioRawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Maximum VCC_PSIO3 is %0d.%03d Volts. \r\n",
	(int)(MaxData), SysMonPsuFractionToInt(MaxData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MIN_SUPPLY5, XSYSMON_PS);
	MinData = XSysMonPsu_VccopsioRawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"The Minimum VCC_PSIO3 is %0d.%03d Volts. \r\n",
	(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	* Read the SUPPLY6 VCC_PSIO0_500 Voltage Data (Current/Maximum/Minimum) from the
	* ADC data registers.
	*/
	VccRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_SUPPLY6, XSYSMON_PS);
	VccData = XSysMonPsu_VccopsioRawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"\r\nThe Current VCC_PSIO0 is %0d.%03d Volts. \r\n",
	(int)(VccData), SysMonPsuFractionToInt(VccData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MAX_SUPPLY6, XSYSMON_PS);
	MaxData = XSysMonPsu_VccopsioRawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Maximum VCC_PSIO0 is %0d.%03d Volts. \r\n",
	(int)(MaxData), SysMonPsuFractionToInt(MaxData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MIN_SUPPLY6, XSYSMON_PS);
	MinData = XSysMonPsu_VccopsioRawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"The Minimum VCC_PSIO0 is %0d.%03d Volts. \r\n",
	(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	* Read the SUPPLY7 VCC_PSIO1_501 Voltage Data (Current/Maximum/Minimum) from the
	* ADC data registers.
	*/
	VccRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_SUPPLY7, XSYSMON_PS);
	VccData = XSysMonPsu_VccopsioRawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"\r\nThe Current VCC_PSIO1 is %0d.%03d Volts. \r\n",
	(int)(VccData), SysMonPsuFractionToInt(VccData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MAX_SUPPLY7, XSYSMON_PS);
	MaxData = XSysMonPsu_VccopsioRawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Maximum VCC_PSIO1 is %0d.%03d Volts. \r\n",
	(int)(MaxData), SysMonPsuFractionToInt(MaxData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MIN_SUPPLY7, XSYSMON_PS);
	MinData = XSysMonPsu_VccopsioRawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"The Minimum VCC_PSIO1 is %0d.%03d Volts. \r\n",
	(int)(MinData), SysMonPsuFractionToInt(MinData));


	/*
	* Read the SUPPLY8 VCC_PSIO2_502 Voltage Data (Current/Maximum/Minimum) from the
	* ADC data registers.
	*/
	VccRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_SUPPLY8, XSYSMON_PS);
	VccData = XSysMonPsu_VccopsioRawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"\r\nThe Current VCC_PSIO2 is %0d.%03d Volts. \r\n",
	(int)(VccData), SysMonPsuFractionToInt(VccData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MAX_SUPPLY8, XSYSMON_PS);
	MaxData = XSysMonPsu_VccopsioRawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Maximum VCC_PSIO2 is %0d.%03d Volts. \r\n",
	(int)(MaxData), SysMonPsuFractionToInt(MaxData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MIN_SUPPLY8, XSYSMON_PS);
	MinData = XSysMonPsu_VccopsioRawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"The Minimum VCC_PSIO2 is %0d.%03d Volts. \r\n",
	(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	* Read the SUPPLY9 PS_MGTRAVCC Voltage Data (Current/Maximum/Minimum) from the
	* ADC data registers.
	*/
	VccRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_SUPPLY9, XSYSMON_PS);
	VccData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"\r\nThe Current PS_MGTRAVCC is %0d.%03d Volts. \r\n",
	(int)(VccData), SysMonPsuFractionToInt(VccData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MAX_SUPPLY9, XSYSMON_PS);
	MaxData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Maximum PS_MGTRAVCC is %0d.%03d Volts. \r\n",
	(int)(MaxData), SysMonPsuFractionToInt(MaxData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MIN_SUPPLY9, XSYSMON_PS);
	MinData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"The Minimum PS_MGTRAVCC is %0d.%03d Volts. \r\n",
	(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	* Read the SUPPLY10 PS_MGTRAVTT Voltage Data (Current/Maximum/Minimum) from the
	* ADC data registers.
	*/
	VccRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_SUPPLY10, XSYSMON_PS);
	VccData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"\r\nThe Current PS_MGTRAVTT is %0d.%03d Volts. \r\n",
	(int)(VccData), SysMonPsuFractionToInt(VccData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MAX_SUPPLY10, XSYSMON_PS);
	MaxData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Maximum PS_MGTRAVTT is %0d.%03d Volts. \r\n",
	(int)(MaxData), SysMonPsuFractionToInt(MaxData));

	VccRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MIN_SUPPLY10, XSYSMON_PS);
	MinData = XSysMonPsu_RawToVoltage(VccRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS,"The Minimum PS_MGTRAVTT is %0d.%03d Volts. \r\n",
	(int)(MinData), SysMonPsuFractionToInt(MinData));

}

/****************************************************************************/
/**
*
* This function runs a test on the System Monitor device using the
* driver APIs.
* This function does the following tasks:
*   - Initiate the System Monitor device driver instance
*   - Run self-test on the device
*   - Setup the sequence registers to continuously monitor on-chip
*     temperature, remote temperature , VCC_PSINTLP,VCC_PSINTFP,VCC_PSAUX
*     VCC_PSDDR_504,VCC_PSIO3_503,VCCAMS,VCC_PSIO0_500,VCC_PSIO1_501,VCC_PSIO2_502
*     PS_MGTRAVCC,PS_MGTRAVTT
*
*   - Setup configuration registers to start the sequence
*
*/

static void SysmonPsInit(const XPfw_Module_t *ModPtr, const u32 *CfgData, u32 Len)
{
	s32 Status;
	XSysMonPsu_Config *ConfigPtr;

	XPfw_Printf(DEBUG_PRINT_ALWAYS,"In SysmonPsInit\r\n");


	/* Load Config for PS-SYSNMON */
	ConfigPtr = XSysMonPsu_LookupConfig(XPAR_XSYSMONPSU_0_DEVICE_ID);
	if (ConfigPtr == NULL) {
	Status = XST_FAILURE;
	XPfw_Printf(DEBUG_ERROR,"SYSMONPSU (MOD-%d): SYSMONPSU LookupConfig failed.\r\n",
	SysmonPsModPtr->ModId);
	goto Done;
	}
	/* Initialize the SysMon driver. */
	Status = XSysMonPsu_CfgInitialize(SysMonInstPtr, ConfigPtr,
	ConfigPtr->BaseAddress);

	if (XST_FAILURE == Status) {
	XPfw_Printf(DEBUG_ERROR,"SYSMONPSU (MOD-%d): Initialization failed.\r\n",
	SysmonPsModPtr->ModId);
	goto Done;
	}

	/* Self Test the System Monitor device */
	Status = XSysMonPsu_SelfTest(SysMonInstPtr);
	if (Status != XST_SUCCESS) {
	goto Done;
	}

	/*
	* Disable the Channel Sequencer before configuring the Sequence
	* registers.
	*/
	XSysMonPsu_SetSequencerMode(SysMonInstPtr, XSM_SEQ_MODE_SAFE, XSYSMON_PS);

	/* Disable all the alarms in the Configuration Register 1. */
	XSysMonPsu_SetAlarmEnables(SysMonInstPtr, 0x0, XSYSMON_PS);

	/*
	* Setup the Averaging to be done for the channels in the
	* Configuration 0 register as 16 samples:
	*/
	XSysMonPsu_SetAvg(SysMonInstPtr, XSM_AVG_16_SAMPLES, XSYSMON_PS);

	/*
	* Enable the averaging on the following channels in the Sequencer
	* registers:
	*   On-chip Temperature, Remote temperature
	*   VCC_PSINTLP,VCC_PSINTFP,VCC_PSAUX
	*   VCC_PSDDR_504,VCCAMS,VCC_PSIO3_503,VCC_PSIO0_500,VCC_PSIO1_501,VCC_PSIO2_502
	*   PS_MGTRAVCC,PS_MGTRAVTT
	*/

	Status =  XSysMonPsu_SetSeqAvgEnables(SysMonInstPtr, XSYSMONPSU_SEQ_CH0_TEMP_MASK |
	XSYSMONPSU_SEQ_CH0_SUP1_MASK | XSYSMONPSU_SEQ_CH0_SUP2_MASK |
	XSYSMONPSU_SEQ_CH0_SUP3_MASK | XSYSMONPSU_SEQ_CH0_SUP4_MASK |
	XSYSMONPSU_SEQ_CH0_SUP5_MASK | XSYSMONPSU_SEQ_CH0_SUP6_MASK |
	((u64)(XSYSMONPSU_SEQ_CH2_VCCAMS_MASK | XSYSMONPSU_SEQ_CH2_TEMP_RMT_MASK |
	XSYSMONPSU_SEQ_CH2_SUP7_MASK | XSYSMONPSU_SEQ_CH2_SUP8_MASK |
	XSYSMONPSU_SEQ_CH2_SUP9_MASK | XSYSMONPSU_SEQ_CH2_SUP10_MASK )) << 32, XSYSMON_PS);
	if (Status != XST_SUCCESS) {
	goto Done;
	}
	/*
	* Enable the following channels in the Sequencer registers:
	*  On-chip Temperature, Remote temperature
	*  VCC_PSINTLP,VCC_PSINTFP,VCC_PSAUX
	*  VCC_PSDDR_504,VCCAMS,VCC_PSIO3_503,VCC_PSIO0_500,VCC_PSIO1_501,VCC_PSIO2_502
	* PS_MGTRAVCC,PS_MGTRAVTT
	*/

	Status =  XSysMonPsu_SetSeqChEnables(SysMonInstPtr, XSYSMONPSU_SEQ_CH0_TEMP_MASK |
	XSYSMONPSU_SEQ_CH0_SUP1_MASK | XSYSMONPSU_SEQ_CH0_SUP2_MASK |
	XSYSMONPSU_SEQ_CH0_SUP3_MASK | XSYSMONPSU_SEQ_CH0_SUP4_MASK |
	XSYSMONPSU_SEQ_CH0_SUP5_MASK | XSYSMONPSU_SEQ_CH0_SUP6_MASK |
	((u64)(XSYSMONPSU_SEQ_CH2_VCCAMS_MASK | XSYSMONPSU_SEQ_CH2_TEMP_RMT_MASK |
	XSYSMONPSU_SEQ_CH2_SUP7_MASK | XSYSMONPSU_SEQ_CH2_SUP8_MASK |
	XSYSMONPSU_SEQ_CH2_SUP9_MASK | XSYSMONPSU_SEQ_CH2_SUP10_MASK )) << 32, XSYSMON_PS);

	if (Status != XST_SUCCESS) {
	goto Done;
	}

	/*
	* Set the ADCCLK frequency equal to 1/32 of System clock for the System
	* Monitor in the Configuration Register 2.
	*/
	XSysMonPsu_SetAdcClkDivisor(SysMonInstPtr, 32, XSYSMON_PS);

	/* Enable the Channel Sequencer in continuous sequencer cycling mode. */
	XSysMonPsu_SetSequencerMode(SysMonInstPtr, XSM_SEQ_MODE_CONTINPASS, XSYSMON_PS);

	Status = XPfw_CoreScheduleTask(SysmonPsModPtr, XPFW_PS_SYSMON_VOLTAGE_TEMP_READ_TIME,
									SysmonPsuPolledExamplePrint);
	if (XST_FAILURE == Status) {
	XPfw_Printf(DEBUG_ERROR,"SYSMONPSU (MOD-%d): Scheduling PS-SYSMON read has failed.",
	SysmonPsModPtr->ModId);
	}
Done:
	return ;
}


void ModSysmonPsInit(void)
{
	SysmonPsModPtr =  XPfw_CoreCreateMod();

	(void)XPfw_CoreSetCfgHandler(SysmonPsModPtr, SysmonPsInit);
}

